/*
 *
 * Copyright (C) 2003 Mekensleep
 *
 *	Mekensleep
 *	24 rue vieille du temple
 *	75004 Paris
 *       licensing@mekensleep.com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Author: Loic Dachary <loic@gnu.org>
 *
 * Inspired by chromium sound support.
 */

#include "AudioOpenAL.h"
#include <maf/sound.h>

#include <stdio.h>
#include <stdlib.h>

#include <AL/al.h>
#include <AL/alc.h>
#include <AL/alut.h>

AudioOpenAL* AudioOpenAL::_instance = 0;

AudioOpenAL::AudioOpenAL() :
  fileNames(NumSoundTypes),
  sounds(NumSoundTypes)
{
  fileNames[Bounce]	= "bounce.sound";
  fileNames[Soundtrack]	= "soundtrack.sound";
	
  context_id	= 0;

  int i;
  for(i = 0; i < NumSoundTypes; i++) {
    buffer[i] = 0;
    source[i] = 0;
    gain[i] = 1.0;
  }
	
  initSound();
}

AudioOpenAL::~AudioOpenAL()
{
  if(context_id)
    alcDestroyContext(context_id);
}

void AudioOpenAL::initSound()
{
  int i;
  ALfloat zeroes[] = { 0.0f, 0.0f,  0.0f };
  ALfloat back[]   = { 0.0f, 0.0f, -1.0f, 0.0f, 1.0f, 0.0f };
	
  ALCdevice *dev;
  dev = alcOpenDevice( NULL );
  context_id = alcCreateContext(dev, NULL);
  alcMakeContextCurrent(context_id);

  alListenerfv(AL_POSITION, zeroes);
  alListenerfv(AL_ORIENTATION, back );
	
  alGenBuffers(NumSoundTypes, buffer);
  alGenSources(NumSoundTypes, source);
	
  loadSounds();
	
  for(i = 0; i < NumSoundTypes; i++) {
    fprintf(stderr, "AudioOpenAL.cpp - init %s\n", fileNames[i].c_str());
    alSourcefv( source[i], AL_POSITION, zeroes );
    alSourcei ( source[i], AL_LOOPING, sounds[i].GetInfo().Looping);
    alSourcei ( source[i], AL_BUFFER, buffer[i] );
    alSourcef ( source[i], AL_GAIN, gain[i]);
    checkError();
  }
	
//  alSourcei    (source[Soundtrack], AL_LOOPING, AL_TRUE);
}
	
void AudioOpenAL::stopMusic()
{
  alSourceStop(source[Soundtrack]);
}

void AudioOpenAL::pauseGameMusic()
{
  alSourcePause(source[Soundtrack]);
}

void AudioOpenAL::playGameMusic()
{
  alSourcePlay(source[Soundtrack]);
}

void AudioOpenAL::loadSounds()
{
  int i;
  ALsizei size, freq, bits;
  ALenum format;
  ALvoid *data;

  for(i = 0; i < NumSoundTypes; i++) {
    sounds[i].Load(fileNames[i]);
    alutLoadWAV(sounds[i].GetInfo().Path.c_str(), &data, &format, &size, &bits, &freq);
    checkError("alutLoadWAV");
    alBufferData (buffer[i], format, data, size, freq);
    checkError("alBufferData");
    free(data);
  }
}

void AudioOpenAL::checkError(const char* message)
{
  ALenum error = alGetError();
  if(error != AL_NO_ERROR) {
    fprintf(stderr, "AudioOpenAL <%s> alGetError() = %s\n", message, alGetString(error) );
  }
}

void AudioOpenAL::playSound(SoundType type, float *pos)
{
  alSourceRewind(source[Bounce]);
  alSourcefv  (source[Bounce], AL_POSITION, pos );
  alSourcePlay(source[Bounce]);
}
