/*
  Underware - Pong demo
  Copyright (C) 2003 Vincent Caron <v.caron@zerodeux.net>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <SDL.h>
#include <GL/gl.h>
#include <GL/glu.h>

#include "PongDrawArea.h"
#include "PongGameState.h"

PongDrawArea::PongDrawArea ()
{
  mpGameState = NULL;

}


void PongDrawArea::LinkGameState (const PongGameState* pState)
{
  mpGameState = pState;
}


void PongDrawArea::DrawGame ()
{
  /* Setup a 2D projection with a [-1, 1] range on both axis
   */
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluOrtho2D(-1.f, 1.f, -1.f, 1.f);
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();

  DrawPaddle(0);
  DrawPaddle(1);
  DrawBall();

  SDL_GL_SwapBuffers( );
}

/* These are half-dimensions of the paddle, it's just easier to compute
 * positions later with these numbers. The screen is 2x2 in size,
 * and not necessarily looks like a square.
 */
const float cPaddleHWidth  = PongGameState::cPaddleHWidth;
const float cPaddleHLength = PongGameState::cPaddleHLength;

void PongDrawArea::DrawPaddle(int Index)
{
  if (!mpGameState) return;

  float x = (1.f - cPaddleHWidth) * (Index ? 1.f : -1.f);
  float y = mpGameState->mPlayerYPos[Index] * (1.f - cPaddleHLength);

  glColor3f(1.f, 1.f, 1.f);
  glBegin(GL_QUADS);
  glVertex2f(x - cPaddleHWidth, y + cPaddleHLength);
  glVertex2f(x - cPaddleHWidth, y - cPaddleHLength);
  glVertex2f(x + cPaddleHWidth, y - cPaddleHLength);
  glVertex2f(x + cPaddleHWidth, y + cPaddleHLength);
  glEnd();
}

const float cBallHSize = PongGameState::cBallHSize;

void PongDrawArea::DrawBall()
{
  if (!mpGameState) return;

  float x = (1.f - 2.f * cPaddleHWidth - cBallHSize) * mpGameState->mBallXPos;
  float y = (1.f - cBallHSize) * mpGameState->mBallYPos;

  glColor3f(1.f, 1.f, 1.f);
  glBegin(GL_QUADS);
  glVertex2f(x - cBallHSize, y + cBallHSize);
  glVertex2f(x - cBallHSize, y - cBallHSize);
  glVertex2f(x + cBallHSize, y - cBallHSize);
  glVertex2f(x + cBallHSize, y + cBallHSize);
  glEnd();
}
